"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const querystring = require("querystring");
const nock = require("nock");
const GET = "GET";
const HEAD = "HEAD";
const POST = "POST";
const PATCH = "PATCH";
const PUT = "PUT";
const DELETE = "DELETE";
const OPTIONS = "OPTIONS";
const heisennock = function heisennock(basePath) {
    return {
        get: (uri) => _createScopeInterceptor(GET, basePath, uri),
        head: (uri) => _createScopeInterceptor(HEAD, basePath, uri),
        post: (uri) => _createScopeInterceptor(POST, basePath, uri),
        patch: (uri) => _createScopeInterceptor(PATCH, basePath, uri),
        put: (uri) => _createScopeInterceptor(PUT, basePath, uri),
        delete: (uri) => _createScopeInterceptor(DELETE, basePath, uri),
        options: (uri) => _createScopeInterceptor(OPTIONS, basePath, uri)
    };
};
heisennock.cleanAll = cleanAll;
exports.default = heisennock;
function cleanAll() {
    nock.cleanAll();
}
exports.cleanAll = cleanAll;
function _createScopeInterceptor(method, basePath, relativeUrl) {
    const params = {
        basePath,
        method,
        relativeUrl,
        times: Infinity
    };
    return _createPartialInterceptor(params);
}
function _createPartialInterceptor(params) {
    function reply(status, body, headers) {
        const interceptor = _buildInterceptor(params);
        interceptor.original.reply(status, body, headers);
        return interceptor;
    }
    function replyWithError(error) {
        const interceptor = _buildInterceptor(params);
        interceptor.original.replyWithError(error);
        return interceptor;
    }
    function replyWithFile(status, filename, headers) {
        const interceptor = _buildInterceptor(params);
        interceptor.original.replyWithFile(status, filename, headers);
        return interceptor;
    }
    function times(times) {
        return _createPartialInterceptor(Object.assign({}, params, { times }));
    }
    return {
        reply,
        replyWithError,
        replyWithFile,
        times
    };
}
function _buildInterceptor(params) {
    const _headers = [];
    const _queries = [];
    const _bodies = [];
    const _urls = [];
    const baseScope = nock(params.basePath);
    baseScope.on("request", ctx => {
        _headers.push(ctx.headers);
        const rawUrl = ctx.path;
        const parsedUrl = url_1.parse(rawUrl, false);
        /* istanbul ignore else */
        if (parsedUrl.pathname) {
            _urls.push(parsedUrl.pathname);
        }
        const qs = querystring.parse(parsedUrl.query);
        _queries.push(qs);
        return true;
    });
    const httpMethod = getNockScopeHTTPMethod(baseScope, params.method);
    const original = httpMethod.call(baseScope, params.relativeUrl, (body) => {
        interceptor.callCount += 1;
        _bodies.push(body);
        return true;
    })
        .query(true)
        .times(params.times);
    const interceptor = {
        original,
        callCount: 0,
        header,
        headers,
        allHeaders,
        query,
        queries,
        body,
        bodies,
        url: url,
        urls,
    };
    return interceptor;
    function header(name) {
        return _getFirstIntercepted(_headers)[name.toLowerCase()];
    }
    function headers(...names) {
        return _pickHeaders(_getFirstIntercepted(_headers), names);
    }
    function allHeaders(...names) {
        return _headers.map(headers => _pickHeaders(headers, names));
    }
    function query() {
        return _getFirstIntercepted(_queries);
    }
    function queries() {
        return _queries.concat();
    }
    function url() {
        return _getFirstIntercepted(_urls);
    }
    function urls() {
        return _urls.concat();
    }
    function body() {
        return _getFirstIntercepted(_bodies);
    }
    function bodies() {
        return _bodies.concat();
    }
}
function _getFirstIntercepted(intercepted) {
    if (!intercepted.length)
        throw new Error("Heisennock Error: No match");
    return intercepted[0];
}
function _pickHeaders(headers, names) {
    const interestingHeaders = {};
    for (const name of names) {
        const key = name.toLowerCase();
        interestingHeaders[key] = headers[key];
    }
    return interestingHeaders;
}
function getNockScopeHTTPMethod(scope, method) {
    switch (method) {
        case GET: return scope.get;
        case HEAD: return scope.head;
        case POST: return scope.post;
        case PATCH: return scope.patch;
        case PUT: return scope.put;
        case DELETE: return scope.delete;
        case OPTIONS: return scope.options;
    }
}
//# sourceMappingURL=heisennock.js.map